/**********************************************************
	Test program for I/O Expander
	for use on PIC32 Devboard
	- MCP23008 I/O Expander
***********************************************************
	Test program writes a byte to the I/O Expander, then
	reads it back. If it matches, the light will turn on.
	The center switch (I2C Enable) must be moved to "on".
***********************************************************
	This program is based upon example I2C code for PIC32 
	microcontrollers provided by Microchip. It utilizes the
	plib.h header file for peripheral function definitions,
	and the i2c_master.c file for I2C functions
***********************************************************/
#include <plib.h>	// This has definitions for EVERY peripheral
#include "..\Devboard_I2C.h"	// Board definitions for I2C

// Configuration bit settings
// SYSCLK = 48 MHz (8MHz FRC / FPLLIDIV * FPLLMUL / FPLLODIV)
// PBCLK = 48 MHz (SYSCLK / FPBDIV)
// Primary Osc w/PLL (XT+,HS+,EC+PLL)
// WDT OFF
// Other options are don't care
#pragma config FPLLMUL = MUL_24, FPLLIDIV = DIV_2, FPLLODIV = DIV_2, FWDTEN = OFF
#pragma config POSCMOD = HS, FNOSC = FRCPLL, FPBDIV = DIV_1
#define SYS_FREQ (48000000L)

#define GetSystemClock()           (SYS_FREQ)
#define GetPeripheralClock()       (SYS_FREQ/1)
#define GetInstructionClock()      (SYS_FREQ)
#define I2C_CLOCK_FREQ             5000

int main (void)
{
	UINT8 i2cData[10];
	I2C_7_BIT_ADDRESS SlaveAddress;
	int Index;
	int DataSz;
	BOOL Acknowledged;
	BOOL Success = TRUE;
	UINT8 i2cbyte;
	
	TRISB= 0x0;		// Port B as outputs
	PORTB= 0x10;	// Turn LED off

	// Set the I2C baudrate
	I2CSetFrequency(I2C_BUS,GetPeripheralClock(),I2C_CLOCK_FREQ);

	// Enable the I2C bus
	I2CEnable(I2C_BUS,TRUE);

	// Initialize the data buffer
	I2C_FORMAT_7_BIT_ADDRESS(SlaveAddress,IOE_ADDR,I2C_WRITE);
	i2cData[0] = SlaveAddress.byte;
	i2cData[1] = IOE_IODIR;
	i2cData[2] = 0x00;
	DataSz = 3;
	
	// Start the transfer to set GPIO as outputs
	if(!StartTransfer(FALSE)) while(1);

	// Transmit all data
	Index = 0;
	while(Success && (Index < DataSz))
	{
		// Transmit a byte
		if(TransmitOneByte(i2cData[Index]))
		{
			// Go to next byte
			Index++;
			// Verify that the byte was acknowledged
			if(!I2CByteWasAcknowledged(I2C_BUS))
			{
				Success = FALSE;
			}
		}
		else Success = FALSE;
	}
	// End the transfer
	StopTransfer();
	if(!Success) while(1);
	
	// Initialize the data buffer
	I2C_FORMAT_7_BIT_ADDRESS(SlaveAddress,IOE_ADDR,I2C_WRITE);
	i2cData[0] = SlaveAddress.byte;
	i2cData[1] = IOE_OLAT;
	i2cData[2] = 0x55;
	DataSz = 3;
	
	// Start the transfer to set OLAT to 0x55
	if(!StartTransfer(FALSE)) while(1);

	// Transmit all data
	Index = 0;
	while(Success && (Index < DataSz))
	{
		// Transmit a byte
		if(TransmitOneByte(i2cData[Index]))
		{
			// Go to next byte
			Index++;
			// Verify that the byte was acknowledged
			if(!I2CByteWasAcknowledged(I2C_BUS))
			{
				Success = FALSE;
			}
		}
		else Success = FALSE;
	}
	// End the transfer
	StopTransfer();
	if(!Success) while(1);
	
	// Read GPIO
	
	// Initialize the data buffer
	I2C_FORMAT_7_BIT_ADDRESS(SlaveAddress,IOE_ADDR,I2C_WRITE);
	i2cData[0] = SlaveAddress.byte;
	i2cData[1] = IOE_GPIO;
	DataSz = 2;
	
	// Start the transfer to set GPIO as addressed register
	if(!StartTransfer(FALSE)) while(1);

	// Transmit all data
	Index = 0;
	while(Success && (Index < DataSz))
	{
		// Transmit a byte
		if(TransmitOneByte(i2cData[Index]))
		{
			// Go to next byte
			Index++;
			// Verify that the byte was acknowledged
			if(!I2CByteWasAcknowledged(I2C_BUS))
			{
				Success = FALSE;
			}
		}
		else Success = FALSE;
	}
	
	if(Success)
	{
      // Send a Repeated Started condition
        if( !StartTransfer(TRUE) )
        {
            while(1);
        }

        // Transmit the address with the READ bit set
        I2C_FORMAT_7_BIT_ADDRESS(SlaveAddress, IOE_ADDR, I2C_READ);
        if (TransmitOneByte(SlaveAddress.byte))
        {
            // Verify that the byte was acknowledged
            if(!I2CByteWasAcknowledged(I2C_BUS))
            {
                Success = FALSE;
            }
        }
        else
        {
            Success = FALSE;
        }
    }
    // Read the data from the desired address
    if(Success)
    {
        if(I2CReceiverEnable(I2C_BUS, TRUE) == I2C_RECEIVE_OVERFLOW)
        {
            DBPRINTF("Error: I2C Receive Overflow\n");
            Success = FALSE;
        }
        else
        {
            while(!I2CReceivedDataIsAvailable(I2C_BUS));
            i2cbyte = I2CGetByte(I2C_BUS);
        }

    }
    // End the transfer (stop here if an error occured)
    StopTransfer();
    if(!Success)
    {
        while(1);
    }
	
	// If we have a match, turn LED on
	if( i2cbyte == 0x55)
	{
		PORTB &= ~(0x10);
	}	
}